﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using asprise_scan_scanner_twain_wia_api;

namespace asprise_imaging_api
{
    /// <summary>
    /// Sample WinForm demostrating Asprise Scan functions.
    /// </summary>
    public partial class SampleScanForm : Form
    {
        private Dictionary<string, string> formSettings = new Dictionary<string,string>();
        private string formSettingFileName = "scan-demo.properties";
        static string PREF_COMBO_SOURCE = "combo-source";
        static string PREF_SHOW_SCANNER_UI = "show-scanner-ui";
        static string PREF_OWN_CODE = "own-code";

        static string lastOwnCode = null;

        List<DemoRecord> demoRecords;
        List<Button> demoSelectButtons = new List<Button>();

        static Font fontSelector = new Font("Arial", 12);

        public SampleScanForm()
        {
            InitializeComponent();
            myInit();
        }

        void myInit()
        {
            formSettings = DemoUtils.readDynamicSettings(formSettingFileName);
            comboBoxSource.SelectedIndex = formSettings.ContainsKey(PREF_COMBO_SOURCE) && 
                formSettings[PREF_COMBO_SOURCE].Trim().Equals("1") ? 1 : 0;
            checkBoxShowScannerUI.Checked = formSettings.ContainsKey(PREF_SHOW_SCANNER_UI) &&
                formSettings[PREF_SHOW_SCANNER_UI].Trim().ToLower().Equals("false") ? false : true;
            lastOwnCode = formSettings.ContainsKey(PREF_OWN_CODE) ? formSettings[PREF_OWN_CODE] : null;

            demoRecords = DemoUtils.loadDemos();
            Console.WriteLine("Demos loaded: " + demoRecords.Count);
            demoSelectButtons.Clear();

            demoSelectorTableLayout.BackColor = Color.White;
            demoSelectorTableLayout.ColumnCount = 1;
            demoSelectorTableLayout.RowCount = demoRecords.Count + 1;

            demoSelectorTableLayout.Controls.Clear();
            demoSelectorTableLayout.RowStyles.Clear();

            demoSelectorTableLayout.AutoSize = false;
            demoSelectorTableLayout.AutoSizeMode = System.Windows.Forms.AutoSizeMode.GrowOnly;

            int row = 0;
            foreach (DemoRecord dr in demoRecords)
            {
                Button button = new Button();
                button.FlatStyle = FlatStyle.Flat;
                button.FlatAppearance.BorderSize = 1;
                button.FlatAppearance.BorderColor = Color.LightGray;
                button.Font = fontSelector;
                button.FlatStyle = FlatStyle.Flat;
                button.Text = dr.getTitle();
                button.TextAlign = ContentAlignment.MiddleLeft;
                Console.WriteLine(dr.getTitle());
                demoSelectButtons.Add(button);
                demoSelectorTableLayout.RowStyles.Add(new RowStyle()); // new RowStyle(SizeType.Absolute, 30));
                demoSelectorTableLayout.Controls.Add(button, 0, row++);
                button.AutoSize = true;
                button.Dock = DockStyle.Fill;
                button.Click += onDemoSelectorButtonClicked;

                ToolTip tooltipForButton = new ToolTip();
                tooltipForButton.SetToolTip(button, dr.getTitle());
            }

            Label filler = new Label();
            demoSelectorTableLayout.RowStyles.Add(new RowStyle(SizeType.Percent, 100));
            demoSelectorTableLayout.Controls.Add(filler, 0, row++);
            filler.Height = 200;
            //filler.AutoSize = true;
            filler.Dock = DockStyle.Fill;

            switchDemo(-1);

            detectDll();
        }

        void onDemoSelectorButtonClicked(object sender, EventArgs e)
        {
            Button selectedButton = (Button)sender;
            int selectedIndex = demoSelectButtons.IndexOf((Button)sender);
            if (!(selectedIndex >= 0))
            {
                Console.WriteLine("Unable to handle: " + sender);
                return;
            }

            foreach (Button button in demoSelectButtons)
            {
                if (button == selectedButton)
                {
                    button.BackColor = Color.LightSkyBlue;
                } else {
                    button.BackColor = SystemColors.ButtonHighlight;
                }
            }

            switchDemo(selectedIndex);
        }

        int currentDemoIndex = -1;
        DemoRecord currentDemoRecord;
        bool currentDemoIsOwnCode = false;

        public void switchDemo(int index)
        {
            if (currentDemoIndex == index) // nothing to do
            {
                return;
            }

            // store last own code
            if (currentDemoIsOwnCode)
            {
                if (!string.IsNullOrEmpty(DemoUtils.stripComments(richTextBoxSourceCode.Text).Trim()))
                {
                    lastOwnCode = richTextBoxSourceCode.Text;
                }
            }

            currentDemoRecord = index < 0 ? null : demoRecords[index];
            currentDemoIsOwnCode = currentDemoRecord != null && currentDemoRecord.getTitle().ToLower().Contains("own code");
            currentDemoIndex = index;
            labelTitleSourceCode.Text = currentDemoRecord == null ? "Source Code" : currentDemoRecord.getTitle();
            buttonExecuteDemo.Enabled = currentDemoRecord != null;
            buttonExecuteDemo.ForeColor = Color.White;
            buttonExecuteDemo.BackColor = currentDemoRecord == null ? Color.Gray : Color.FromArgb(0, 51, 153);
            richTextBoxSourceCode.Text = "";
            if (currentDemoRecord != null)
            {
                string textToDisplay = currentDemoRecord.getCodeDisplay();
                if (currentDemoIsOwnCode && !string.IsNullOrEmpty(lastOwnCode))
                {
                    textToDisplay = lastOwnCode;
                }

                // Console.WriteLine(textToDisplay);
                using (StringReader reader = new StringReader(textToDisplay))
                {
                    string line = null;
                    while ((line = reader.ReadLine()) != null)
                    {
                        line = line.TrimEnd();
                        if (line.EndsWith("//*"))
                        {
                            line = line.Substring(0, line.Length - 3);
                            AppendText(richTextBoxSourceCode, line, Color.Black, Color.FromArgb(255, 242, 157));
                        }
                        else
                        {
                            AppendText(richTextBoxSourceCode, line, Color.Black, Color.Empty);
                        }
                    }
                }
            }
        }

        public static void AppendText(RichTextBox box, string text, Color color, Color bgColor, bool AddNewLine = true)
        {
            // parse comments
            int posComment = text.IndexOf("//");
            if (posComment >= 0 && !(posComment > 0 && text[posComment - 1] == ':'))
            {
                AppendTextParseStrings(box, text.Substring(0, posComment), color, bgColor, false);
                doAppendText(box, text.Substring(posComment), Color.Green, bgColor, AddNewLine);
            }
            else
            {
                AppendTextParseStrings(box, text, color, bgColor, AddNewLine);
            }
        }

        static void AppendTextParseStrings(RichTextBox box, string text, Color color, Color bgColor, bool AddNewLine)
        {
            if (text.Length == 0)
            {
                doAppendText(box, text, color, bgColor, AddNewLine);
                return;
            }

            // parse strings
            int cursor = 0;
            int quote1 = -1;
            int quote2 = -1;
            
            while (cursor < text.Length)
            {
                quote1 = text.IndexOf('"', cursor);
                while(quote1 > 0 && text[quote1-1] == '\\') { // ignore escaped quote, \"
                    quote1 = text.IndexOf('"', quote1 + 1);
                }

                if (quote1 > 0 && text.Length > quote1 + 1)
                {
                    quote2 = text.IndexOf('"', quote1 + 1);
                    while(quote2 > 0 && text[quote2-1] == '\\') { // ignore escaped quote, \"
                        quote2 = text.IndexOf('"', quote2 + 1);
                    }
                }
                else
                {
                    quote2 = -1;
                }

                if (quote2 < 0) // not found
                {
                    doAppendText(box, text.Substring(cursor), color, bgColor, AddNewLine);
                    break;
                }
                else
                {
                    doAppendText(box, text.Substring(cursor, quote1 - cursor), color, bgColor, false);
                    doAppendText(box, text.Substring(quote1, quote2 - quote1 + 1), Color.Purple, bgColor, false);
                    cursor = quote2 + 1;
                }
            }
        }

        static void doAppendText(RichTextBox box, string text, Color color, Color bgColor, bool AddNewLine = true)
        {
            if (AddNewLine)
            {
                text += Environment.NewLine;
            }

            box.SelectionStart = box.TextLength;
            box.SelectionLength = 0;

            box.SelectionColor = color;
            if (bgColor != Color.Empty) {
                box.SelectionBackColor = bgColor;
            }
            box.AppendText(text);
            box.SelectionColor = box.ForeColor;
        }

        private void buttonExecuteDemo_Click(object sender, EventArgs e)
        {
            if (currentDemoRecord == null)
            {
                MessageBox.Show("Please select a demo first.");
                return;
            }

            string code = currentDemoIsOwnCode ? richTextBoxSourceCode.Text : currentDemoRecord.getCodeRaw();
            Result result = runRequest(DemoUtils.stripComments(code), comboBoxSource.SelectedIndex == 0 ? "select" : "default", checkBoxShowScannerUI.Checked, true);

            try
            {
                formSettings[PREF_COMBO_SOURCE] = comboBoxSource.SelectedIndex.ToString();
                formSettings[PREF_SHOW_SCANNER_UI] = checkBoxShowScannerUI.Checked ? "true" : "false";
                if (currentDemoIsOwnCode)
                {
                    formSettings[PREF_OWN_CODE] = code;
                }
                DemoUtils.saveDynamicSettings(formSettings, formSettingFileName);
            }
            catch(Exception prefEx)
            {
                Console.WriteLine(prefEx.ToString());
            }
        }

        Result runRequest(string code, string source = "select", bool scannerUi = true, bool modalUi = true)
        {
            flowLayoutPanelResult.Controls.Clear();
            Label label = new Label();
            label.AutoSize = true;
            label.Text = "Executing demo ...";
            flowLayoutPanelResult.Controls.Add(label);

            Console.WriteLine("Request: \n" + code);
            AspriseImaging imaging = new AspriseImaging(this);
            Result result = null;
            try
            {
                result = imaging.Scan(code, source, scannerUi, modalUi);
                displayResult(result);
            }
            catch (Exception e)
            {
                displayResult(result, e);
            }
            
            return result;
        }

        void displayResult(Result result, Exception e = null)
        {
            flowLayoutPanelResult.Controls.Clear();
            if (result == null) // user cancels
            {
                Label label = new Label();
                label.AutoSize = true;
                label.Text = e == null ? "<User cancels>" : e.GetType().Name + ": " + e.Message;
                flowLayoutPanelResult.Controls.Add(label);
            }
            else
            {
                List<string> barcodes = result == null ? null : result.GetBarcodes();
                if (barcodes != null && barcodes.Count > 0)
                {
                    TextBox textBox = new TextBox();
                    textBox.Multiline = true;
                    textBox.ScrollBars = ScrollBars.Horizontal;
                    textBox.AcceptsReturn = true;
                    textBox.WordWrap = false;
                    textBox.Text = string.Join("\r\n", barcodes == null ? new string[0] : barcodes.ToArray());
                    textBox.Width = 200;
                    textBox.Dock = DockStyle.Left;
                    flowLayoutPanelResult.Controls.Add(textBox);
                }

                List<ResultImageItem> imageItems = result.GetImageItems();

                List<Image> images = result.GetImages(false);
                if (images != null && images.Count > 0)
                {
                    for (int i = 0; i < images.Count; i++)
                    {
                        SampleScanResultItemRender render = new SampleScanResultItemRender(images[i],
                            imageItems != null && imageItems.Count > i ? imageItems[i] : null);
                        flowLayoutPanelResult.Controls.Add(render);
                    }
                }

                List<string> imgFiles = result.GetImageFiles();
                if (imgFiles != null && imgFiles.Count > 0)
                {
                    for (int i = 0; i < imgFiles.Count; i++)
                    {
                        SampleScanResultItemRender render = new SampleScanResultItemRender(imgFiles[i],
                            imageItems != null && imageItems.Count > i ? imageItems[i] : null);
                        flowLayoutPanelResult.Controls.Add(render);
                    }
                }

                String serverResponse = result.GetUploadResponse();
                if (! string.IsNullOrEmpty(serverResponse))
                {
                    string fileHtml = Path.GetTempFileName() + ".htm";
                    using (StreamWriter sw = new StreamWriter(fileHtml))
                    {
                        sw.Write(serverResponse);
                    }
                    SampleScanResultItemRender render = new SampleScanResultItemRender(fileHtml, imageItems != null && imageItems.Count > 0 ? imageItems[0] : null);
                    flowLayoutPanelResult.Controls.Add(render);
                }
            }
        }

        private void buttonVisitProductPage_Click(object sender, EventArgs e)
        {
            System.Diagnostics.Process.Start("http://asprise.com/c%23-vb.net-scanner-api/twain-wia-scan-pdf-library-overview.html");
        }

        private void linkLabelReadDevGuide_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            System.Diagnostics.Process.Start("http://asprise.com/c%23-vb.net-scanner-api/twain-wia-document-scan-library-sdk-samples-docs.html");
        }

        public void detectDll()
        {

            bool browsed = false;
            int count = 0;

            // Let user browse the ocr dll if it is not found in PATH.
            while (true)
            {
                string dllFilePath = AspriseImaging.getDll();
                if (dllFilePath == null)
                {
                    Console.WriteLine("Asprise imaging dll (" + AspriseImaging.getImgDllName() + ") is not found in PATH.");
                    OpenFileDialog fileDialog = new OpenFileDialog();
                    fileDialog.RestoreDirectory = true;
                    fileDialog.Title = "Please select " + AspriseImaging.getImgDllName();
                    fileDialog.FileName = AspriseImaging.getImgDllName();
                    if (fileDialog.ShowDialog(this) == DialogResult.OK)
                    {
                        Console.WriteLine("User browsed dll: " + fileDialog.FileName);
                        AspriseImaging.addToSystemPath(Path.GetDirectoryName(fileDialog.FileName));
                        browsed = true;
                    }
                    count++;
                    if (count == 2)
                    {
                        MessageBox.Show("Asprise Imaging DLL (" + AspriseImaging.getImgDllName() + ") not found. ", "Error");
                        break;
                    }
                }
                else
                {
                    Console.WriteLine("OCR dll found: " + dllFilePath);
                    if (browsed)
                    {
                        Console.WriteLine("Please consider copy " + dllFilePath + " to directory: " +
                            Directory.GetCurrentDirectory());
                    }

                    Console.WriteLine(AspriseImaging.GetLibraryVersion());
                    labelSystemInfo.Text = (AspriseImaging.Is64BitProcess ? "[x64]" : "[x32]") + " " + AspriseImaging.GetLibraryVersion();
                    ToolTip tooltip = new ToolTip();
                    tooltip.SetToolTip(labelSystemInfo, "Built on " + AspriseImaging.GetLibraryBuildInfo());
                    break;
                }
            }
        } // detectDll
    }
}
